/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.saml;

import java.io.InputStream;
import java.util.Properties;
import java.security.Principal;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.jboss.security.SimplePrincipal;

//xml classes
import org.w3c.dom.*;

//jboss-security-common library
import org.jboss.mx.util.MBeanProxy;
import org.jboss.mx.util.MBeanServerLocator;
import org.jboss.security.sso.util.SSOUtil;
import org.jboss.security.util.Tools;

/**
 * SSOManager is an implementation of the SSOManagerService that can be managed via a jmx-console
 * 
 * @author Sohil Shah - sohil.shah@jboss.com
 *
 */
public class SSOManager implements SSOManagerService
{
    /**
     * 
     */
    private static final Logger log = Logger.getLogger(SSOManager.class);
    
    /**
     * 
     */
	private SingleSignOn processor = null;
    private String confLocation = null;
    
    //xml constants
    private static final String sso_processor = "sso-processor";
    private static final String processorTag = "processor";
    private static final String classAttr = "class";
    private static final String property = "property";
    private static final String name = "name";
    
    
	
    /**
     * Initializer
     *
     */
	public SSOManager()
	{
	}
                        
    //--------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Starts the JMX service
     */
    public void start()
    {        
        InputStream is = null;
        try
        {
            if(!this.confLocation.startsWith("/"))
            {
                this.confLocation = "/" + this.confLocation;
            }
            
            is = SSOManager.class.getResourceAsStream(this.confLocation);
                        
            //load the configuration
            Document configDocument = Tools.parseXml(is);
            
            //process the xml
            this.loadConfiguration(configDocument);
            
            log.info("SSOProcessor "+this.getProcessor()+" was successfully registered.....");
            log.info("SSOManager service successfully started...........");
        }
        catch(Exception e)
        {
            log.error(SSOManager.class.getName(),e);
            this.stop();
        }
        finally
        {
            if(is!=null)
            {
                try{is.close();}catch(Exception e){}
            }            
        }
    }
    
    /**
     * Stops the JMX service
     *
     */
    public void stop()
    {
        this.processor = null;
        this.confLocation = null;
    }
    
    /**
     * Sets the location of the configuration file for this service
     *  
     * @param confLocation
     */
    public void setConf(String confLocation)
    {
        this.confLocation = confLocation;
    }
    
    /**
     * Returns the SingleSignOn implementation registered with the SSO system
     * 
     * @return
     */
    public SingleSignOn getProcessor()
    {
        return this.processor;
    }
    
    /**
     * 
     * @param document
     * @throws Exception
     */
    private void loadConfiguration(Document document) throws Exception
    {
        NodeList ssoProcessorNodes = document.getDocumentElement().getElementsByTagName(SSOManager.sso_processor);
        if(ssoProcessorNodes!=null && ssoProcessorNodes.getLength()>0)
        {
            Element ssoProcessor = (Element)ssoProcessorNodes.item(0);             
            Element processorElement = (Element)ssoProcessor.getElementsByTagName(SSOManager.processorTag).item(0);            
            String processorClass = processorElement.getAttribute(SSOManager.classAttr);            
            this.processor = (SingleSignOn)Class.forName(processorClass).newInstance();
            
            //process any properties associated with the processor being plugged-in
            NodeList propertyNodes = processorElement.getElementsByTagName(SSOManager.property);
            if(propertyNodes!=null && propertyNodes.getLength()>0)
            {
                Properties properties = new Properties();
                for(int i=0;i<propertyNodes.getLength();i++)
                {
                    Element propertyNode = (Element)propertyNodes.item(i);
                    String name = propertyNode.getAttribute(SSOManager.name);
                    String value = propertyNode.getFirstChild().getNodeValue();
                    properties.put(name,value);
                }
                this.processor.setProperties(properties);
            }
        }
    }
    
    
    //-------static utility based methods---------------------------------------------------------------------------------------------------
    /**
     * A utility method used to find the SingleSignOn Processor
     * 
     * @return
     */
	public static SingleSignOn findProcessor() throws SSOException
    {
        try
        {
    	    SingleSignOn sso = null;
            
    	    //get the registered ssoManager service mbean
            MBeanServer mbeanServer = MBeanServerLocator.locateJBoss();
            SSOManagerService ssoManager = (SSOManagerService)MBeanProxy.get
            (SSOManagerService.class,
            new ObjectName("jboss.sso:service=SSOManager"), 
            mbeanServer);
            
            sso = ssoManager.getProcessor();
                    
            return sso;
        }
        catch(Exception e)
        {
            throw new SSOException(e);
        }
    }
    
    /**
     * A utility method used to cleanup the SSO token when received as a http cookie
     * 
     * @param ssoToken
     * @return
     */
    public static String cleanToken(String ssoToken)
    {
        return SSOUtil.cleanSSOToken(ssoToken);
    }
    
    /**
     * This method is used by the application to notify the SSOEngine that a particular user was manually authenticated by its authentication
     * system. This will be typically used by applications using non-JAAS (non-container) based authentication mechanisms. This API call is not
     * needed by applications that used container provided mechanisms like JAAS
     * 
     * @param request
     * @param user
     * @throws IllegalArgumentException
     */
    public static void processManualLoginNotification(HttpServletRequest request,String user)
    {
        if(user!=null)
        {
            SSOUser ssoUser = new SSOUser(user,null);
            request.getSession(false).setAttribute(SingleSignOn.SSO_USER,ssoUser);
        }
        else
        {
            throw new IllegalArgumentException();
        }
    }
    
    /**
     * This is method is used by the application to extract the Principal of the user authenticated into the system by the Single Sign On Engine.
     * 
     * This will be typically used by applications using non-JAAS (non-container) based authentication mechanisms. For applications using container managed
     * authentication mechanisms like JAAS, the recommended approach is using a HttpServletRequest.getUserPrincipal call. 
     * 
     * @param request
     * @return
     */
    public static Principal getUserPrincipal(HttpServletRequest request)
    {
        Principal principal = null;
        
        SSOUser ssoUser = (SSOUser)request.getSession(false).getAttribute(SingleSignOn.SSO_USER);        
        if(ssoUser!=null)
        {
            principal = new SimplePrincipal(ssoUser.getUserName());
        }
        
        return principal;
    }
}
